

#include "common.h"
#include "message.h"

#include "merge.h"

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "stats.h"
#include "transfer.h"
#include "buffer.h"

extern pid_t mypid; 

extern int LinkNum;

extern int * StatsMem[NUMCOUNTERS];
extern int * RatesMem[NUMCOUNTERS];

extern int * MERGE_RunOptions;
extern int * MERGE_SyncStep;
extern int * MERGE_Verbose;

extern unsigned long long WRTimeF;
extern unsigned long long timestampF;

extern unsigned long WRTime0027;
extern unsigned long WRTime2847;
extern unsigned long WRTime4863;

extern unsigned long Time0027;
extern unsigned long Time2847;
extern unsigned long Time4863;

extern int BlockSequence[];
extern char * data_buffer[];
extern char * input_buffer;

extern STATUS_TABLE *status_table;
extern LINK_TABLE *link_table[];
extern BUFFER_TABLE *buffer_table;

extern void * shm_bufferarea[];

unsigned int MyCounter = 0;

unsigned int LastCounter = 0;
unsigned int LinkCounter = 0;
unsigned int EventNum = 0;

char spinner[4] = "-\\|/";
int spin = 0;

int data_restructure(char *, char *, int, int *);
int datacopy(NewItDesc_Ptr, ItDesc_Ptr, int);


int processAIDADATAbuffer(ItDesc_Ptr, NewItDesc_Ptr, int);

void byteswop32(char*, int);



/*
   link_server.c  calls receiveDATAbuffer
    
   handle_buffer.c contains receiveDATAbuffer

   receiveDATAbuffer  calls data_restructure  &  store_buffer
         data_restructure calls datacopy calls processDATAbuffer
*/


void queueDATAbuffer (int ID, int endian, char * ptr, int l)
{
      DATA_HEADER* data_header;
      BUFFER_HEADER* baseaddress;
      int * BufferL;
      int i, j;
      int rc;

        data_header = (DATA_HEADER*) (ptr-sizeof(HEADER));

//    construct 24 byte block header by overwritting the 32 byte data transfer header

           memcpy (data_header->header_id,"EBYEDATA",8);
           data_header->header_sequence = 0;
           data_header->header_stream = (ID << 8) + 1;
           data_header->header_tape = 0;
           data_header->header_MyEndian = 1;
           data_header->header_DataEndian = endian;
           data_header->header_dataLen = l;

           memcpy (ptr-8,"AAAAAAAA",8);

       sprintf(message_buffer, "store_buffer: sending %d bytes\n", l);    report_message(MSG_TRACE);

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)data_header;

       printf("queueDATAbuffer #1 len=%d\n",l);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

      baseaddress = (BUFFER_HEADER *) (char*) shm_bufferarea[ID];
                     
//    set the age of the buffer to be written as zero

         i = baseaddress->buffer_next;
         baseaddress->buffer_age[i] = 0;
                          
//    set the age of the buffer and increment the age and the next buffer

         baseaddress->buffer_age[i] = baseaddress->buffer_currentage;
         baseaddress->buffer_currentage++;
                     
         baseaddress->buffer_next = ((baseaddress->buffer_next + 1) & (NBLOCKS - 1));

      if ((*MERGE_RunOptions & 1) != 0) {  

//    signal MERGE that a buffer is waiting

          rc = queue_data_buffer(ID, link_table[ID]->link_buffer_index);
          if (rc != 0) {
             sprintf(message_buffer, "error %d in queue_data_buffer\n", rc);   report_message(MSG_ERROR);
          }

          wakeup_merge();
         (*StatsMem[WAKEUPMERGE])++;           /* counter for number of data buffers */
         (*StatsMem[WAKEUPMERGE+((LinkNum+1)*MAXCOUNTERS)])++;

       } else {

//    test mode - discard buffer

          j = ID*N_DATA_BUFFERS;
          i = link_table[ID]->link_buffer_index;
          buffer_table->buffer_status[j+i] = DONE;

          rc = release_data_buffer(ID, link_table[ID]->link_buffer_index);
       }

    sprintf(message_buffer, "store_buffer #2  %d %d >\n", ID, l);   report_message(MSG_DEBUG);

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)baseaddress;

       printf("queueDATAbuffer #2 len=%d\n",l);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

}

int receiveDATAbuffer(int ID, int endian, char * ptr, int len)
{
    char * iptr;
    char * optr;
    HEADER *ip;
    HEADER op;
    int index;
      int * BufferL;
      int i, j;
      int rc;

    (*StatsMem[LBUFFERS])++;           /* counter for number of data buffers */
    (*StatsMem[LBUFFERS+((LinkNum+1)*MAXCOUNTERS)])++;

    (*StatsMem[DATALEN]) = (*StatsMem[DATALEN]) + ((len+512)/1024);      /* counter for data in 1Kbyte units */
    (*StatsMem[DATALEN+((LinkNum+1)*MAXCOUNTERS)]) = (*StatsMem[DATALEN+((LinkNum+1)*MAXCOUNTERS)]) + ((len+512)/1024); 

     iptr = &input_buffer[sizeof(HEADER)];

//    check block sequence number

     ip = (HEADER*) (ptr-sizeof(HEADER));

     memcpy(&op, ip, sizeof(HEADER));

     if (ntohl(ip->hdr_sequence) == BlockSequence[ID]) {
         BlockSequence[ID]++;   // good
     } else {
         sprintf(message_buffer, "block sequence error: ID %d, expected %d; received %d", ID, BlockSequence[ID], ip->hdr_sequence);
         report_message(MSG_WARNING);
         BlockSequence[ID] = ntohl(ip->hdr_sequence)+1;
     }

     if (ntohl(ip->hdr_datalength) != len) { sprintf(message_buffer, "check#1: %d %d",ntohl(ip->hdr_datalength),len); report_message(MSG_WARNING);}
     if (len + 32 > ntohl(ip->hdr_blocklength)) { sprintf(message_buffer, "check#2: %d %d",ntohl(ip->hdr_blocklength),len); report_message(MSG_WARNING);}
     if (ntohl(ip->hdr_id1) != HDR_ID1) { sprintf(message_buffer, "check#3: 0x%x",ntohl(ip->hdr_id1)); report_message(MSG_WARNING);}
     if (ntohl(ip->hdr_id2) != HDR_ID2) { sprintf(message_buffer, "check#4: 0x%x",ntohl(ip->hdr_id2)); report_message(MSG_WARNING);}

     if (len > ((64*1024)-32)) { sprintf(message_buffer, "%d: Unexpected block length %d",ID,len); report_message(MSG_ERROR); return(0);}

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)ip;

       printf("receiveDATAbuffer #1 len=%d\n",len);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

        sprintf(message_buffer, "data_transfer #1.1 len=%d  >\n", len);   report_message(MSG_DEBUG);

   index = obtain_data_buffer(ID);        /* get buffer for incoming data */
   link_table[ID]->link_buffer_index = index;
   if (index == -1) return -1;
   data_buffer[ID] = (char *) ((char *)buffer_table->buffer_baseaddress[ID]  + (index * status_table->data_buffer_size * 1024));

    optr = &data_buffer[ID][sizeof (HEADER)];

     len = data_restructure(optr, iptr, len, &endian); /*    maybe swap for TDR format   */

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)optr;

       printf("receiveDATAbuffer #2 len=%d\n",len);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

     sprintf(message_buffer, "receiveDATAbuffer #2.1 len=%d  >\n", len);   report_message(MSG_DEBUG);

     queueDATAbuffer(ID, endian, optr, len);

//   if (verbose > 1) {
       printf ("%c\b", spinner[spin]); fflush(NULL);
             if (++spin == 4) spin=0;
//   }
 
      return 0;
}

int datacopy(NewItDesc_Ptr op, const ItDesc_Ptr ip, int l) 
{
    ItDesc_Ptr ps;
    NewItDesc_Ptr pd;
    int len, ret;
      int * BufferL;
      int i, j;

   len = l / sizeof (ItDesc);   /* this is 64 bits */
   ps = ip;   /*  input data buffer  */
   pd = op;   /*  output data buffer  (in internal format)  */
   ret = 0;

     sprintf(message_buffer, "%d datacopy.1 <  len %d pd= 0x%llx  ps= 0x%llx",mypid, len, pd, ps);   report_message(MSG_TRACE);

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)ip;

       printf("datacopy #1 len=%d\n",len);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

/*
      process the data buffer (which may be project specific)
      extract the time information

      returns output length (NewItDesc)
*/

    ret = processAIDADATAbuffer (ps, pd, len);

    if (*MERGE_Verbose > 2) {
       BufferL = (int*)op;

       printf("datacopy #2 len=%d\n",ret);
       printf(">\n");

       for (j = 0; j < 8; j++) {
         for (i = 0; i < 16; i++)
         {
            printf("%08lx ", *BufferL++);
         }
         printf("\n");
       }
    }

     sprintf(message_buffer, "%d datacopy.2 > len %d",mypid, ret);   report_message(MSG_TRACE);

   return (ret * sizeof(NewItDesc));   /* this is bytes */
}

int data_restructure(char* op, char* ip, int l, int *pEndian)
{
   int len;
   int i, j;

   if (*pEndian != 1) {
        byteswop32(ip, l);        /*    for TDR format   */
        *pEndian = 1;
   }

   sprintf(message_buffer, "data_restructure length in: %d (bytes)", l);   report_message(MSG_DEBUG);

// We now have correct data - still in the input buffer - ip, copy it to new structured output buffer

   len = datacopy((NewItDesc_Ptr)op, (ItDesc_Ptr)ip, l);

   sprintf(message_buffer, "data_restructure length out: %d (bytes)", len);   report_message(MSG_DEBUG);

   return len;
}

#include "processAIDADATAbuffer.c"

void byteswop32(char* p, int l) 
{
    int len;
    const char *ps;
    char *pd;
    char b0, b1, b2, b3;

   len = l/4;
   ps = p;
   pd = p;


   while (len > 0)
   {
       b0 = *ps++;
       b1 = *ps++;
       b2 = *ps++;
       b3 = *ps++;

       *pd++ = b3;
       *pd++ = b2;
       *pd++ = b1;
       *pd++ = b0;
   
       len--;
   }
}


